# ABSTRACT: Set the default stack

package Pinto::Action::Default;

use Moose;
use MooseX::Types::Moose qw(Bool);
use MooseX::MarkAsMethods (autoclean => 1);

use Pinto::Types qw(StackName StackObject);

#------------------------------------------------------------------------------

our $VERSION = '0.065_02'; # VERSION

#------------------------------------------------------------------------------

extends qw( Pinto::Action );

#------------------------------------------------------------------------------

with qw( Pinto::Role::Transactional );

#------------------------------------------------------------------------------

has stack => (
    is       => 'ro',
    isa      => StackName | StackObject,
);


has none => (
	is      => 'ro',
	isa     => Bool,
	default => 0,
);

#------------------------------------------------------------------------------

sub execute {
    my ($self) = @_;

   	if ($self->none) {
   		my $default_stack = $self->repo->get_stack;
      return $self->result if not defined $default_stack;
      $default_stack->unmark_as_default;

   	}
   	else {
    	my $stack = $self->repo->get_stack($self->stack);
      return $self->result if $stack->is_default;
      $stack->mark_as_default;
	  }
  
    return $self->result->changed;
}

#------------------------------------------------------------------------------

__PACKAGE__->meta->make_immutable;

#------------------------------------------------------------------------------

1;

__END__

=pod

=for :stopwords Jeffrey Ryan Thalhammer Imaginative Software Systems

=head1 NAME

Pinto::Action::Default - Set the default stack

=head1 VERSION

version 0.065_02

=head1 AUTHOR

Jeffrey Ryan Thalhammer <jeff@imaginative-software.com>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2011 by Imaginative Software Systems.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
